@@ -1,7 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2003 CodeFactory AB
- * Copyright (C) 2003 Mikael Hallendal <micke@imendio.com>
+ * Copyright (C) 2003 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,11 +21,9 @@
 #ifndef __GOSSIP_ADD_CONTACT_H__
 #define __GOSSIP_ADD_CONTACT_H__
 
-#include <loudmouth/loudmouth.h>
-#include "gossip-jid.h"
+#include <glib.h>
 
-void  gossip_add_contact_new (LmConnection *connection,
-			      GossipJID    *jid);
+void  gossip_add_contact_new (const gchar *id);
 
 #endif /* __GOSSIP_ADD_CONTACT_H__ */
 
