@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2003 Imendio HB
+ * Copyright (C) 2003 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -25,14 +25,24 @@
 #include <time.h>
 #include <gtk/gtk.h>
 #include <gconf/gconf-client.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <libgnome/gnome-url.h>
-#include "gossip-utils.h"
+
+#include <libgossip/gossip-time.h>
+#include <libgossip/gossip-utils.h>
+#include <libgossip/gossip-session.h>
+#include <libgossip/gossip-chatroom-provider.h>
+
 #include "gossip-chat-view.h"
+#include "gossip-app.h"
 
 /* Number of seconds between timestamps when using normal mode, 5 minutes */
 #define TIMESTAMP_INTERVAL 300
 
+/* Maximum lines in any chat buffer, see bug #141292 */
+#define MAX_LINES 500
+
+
 struct _GossipChatViewPriv {
 	GtkTextBuffer *buffer;
 
@@ -40,6 +50,7 @@ struct _GossipChatViewPriv {
 	GDate         *last_datestamp;
 };
 
+
 typedef enum {
 	GOSSIP_SMILEY_NORMAL,      /*  :)   */
 	GOSSIP_SMILEY_WINK,        /*  ;)   */
@@ -56,12 +67,14 @@ typedef enum {
 	NUM_SMILEYS
 } GossipSmiley;
 
+
 typedef struct {
 	GossipSmiley  smiley;
 	gchar        *pattern;
 	gint          index;
 } GossipSmileyPattern;
 
+
 static GossipSmileyPattern smileys[] = {
 	{ GOSSIP_SMILEY_NORMAL,     ":)",  0 },
 	{ GOSSIP_SMILEY_WINK,       ";)",  0 },
@@ -87,90 +100,88 @@ static GossipSmileyPattern smileys[] = {
 	{ GOSSIP_SMILEY_SHOCKED,     ":o", 0 },
 	{ GOSSIP_SMILEY_SHOCKED,     ":O", 0 },
 	{ GOSSIP_SMILEY_COOL,        "8)", 0 },
-	{ GOSSIP_SMILEY_COOL,        "B)", 0 }
+	{ GOSSIP_SMILEY_COOL,        "B)", 0 },
+
+	/* backward smiley's */
+	{ GOSSIP_SMILEY_NORMAL,     "(:",  0 },
+	{ GOSSIP_SMILEY_WINK,       "(;",  0 },
+	{ GOSSIP_SMILEY_WINK,       "(-;", 0 },
+	{ GOSSIP_SMILEY_BIGEYE,     "(=",  0 },
+	{ GOSSIP_SMILEY_NOSE,       "(-:", 0 },
+	{ GOSSIP_SMILEY_CRY,        ")':", 0 },
+	{ GOSSIP_SMILEY_SAD,        "):",  0 },
+	{ GOSSIP_SMILEY_SAD,        ")-:", 0 },
+	{ GOSSIP_SMILEY_SCEPTICAL,  "/:",  0 },
+	{ GOSSIP_SMILEY_SCEPTICAL,  "//:",  0 },
+	{ GOSSIP_SMILEY_INDIFFERENT, "|:", 0 },
+	{ GOSSIP_SMILEY_TOUNGE,      "d:", 0 },
+	{ GOSSIP_SMILEY_TOUNGE,      "d-:", 0 },
+	{ GOSSIP_SMILEY_TOUNGE,      "d;", 0 },
+	{ GOSSIP_SMILEY_TOUNGE,      "d-;", 0 },
+	{ GOSSIP_SMILEY_SHOCKED,     "o:", 0 },
+	{ GOSSIP_SMILEY_SHOCKED,     "O:", 0 },
+	{ GOSSIP_SMILEY_COOL,        "(8", 0 },
+	{ GOSSIP_SMILEY_COOL,        "(B", 0 }
+	
 };
 
+
 static gint num_smileys = G_N_ELEMENTS (smileys);
 
-static void       chat_view_class_init      (GossipChatViewClass *klass);
-static void       chat_view_init            (GossipChatView      *view);
-static void       chat_view_finalize        (GObject             *object);
-static void       chat_view_size_allocate   (GtkWidget           *widget,
-					     GtkAllocation       *alloc);
-static void       chat_view_setup_tags      (GossipChatView      *view);
-static void       chat_view_populate_popup  (GossipChatView      *view,
-					     GtkMenu             *menu,
-					     gpointer             user_data); 
-static gboolean   chat_view_event_cb        (GossipChatView      *view,
-					     GdkEventMotion      *event,
-					     GtkTextTag          *tag);
-static gboolean   chat_view_url_event_cb    (GtkTextTag          *tag,
-					     GObject             *object,
-					     GdkEvent            *event,
-					     GtkTextIter         *iter,
-					     GtkTextBuffer       *buffer);
-
-static void       chat_view_open_address    (const gchar         *url);
-
-
-static void       chat_view_open_address_cb (GtkMenuItem         *menuitem,
-					     const gchar         *url);
-
-static void       chat_view_copy_address_cb (GtkMenuItem         *menuitem,
-					     const gchar         *url);
-static void       chat_view_realize_cb      (GossipChatView      *widget,
-					     gpointer             data);
-
-static void       chat_view_clear_view_cb   (GtkMenuItem *menuitem,
-					     GossipChatView *view);
-
-static void       
-chat_view_insert_text_with_emoticons        (GtkTextBuffer       *buf,
-					     GtkTextIter         *iter, 
-					     const gchar         *str);
-
-static GdkPixbuf *chat_view_get_smiley       (GossipSmiley          smiley);
-
-static void              
-chat_view_maybe_append_timestamp             (GossipChatView *view,
-					      const gchar    *time_str);
-static void
-chat_view_maybe_append_datestamp             (GossipChatView *view);
-
-static gboolean chat_view_is_scrolled_down   (GossipChatView *view);
+static void       gossip_chat_view_class_init          (GossipChatViewClass      *klass);
+static void       gossip_chat_view_init                (GossipChatView           *view);
+static void       chat_view_finalize                   (GObject                  *object);
+static void       chat_view_size_allocate              (GtkWidget                *widget,
+							GtkAllocation            *alloc);
+static void       chat_view_setup_tags                 (GossipChatView           *view);
+static void       chat_view_populate_popup             (GossipChatView           *view,
+							GtkMenu                  *menu,
+							gpointer                  user_data);
+static void       chat_view_buffer_changed_cb          (GtkTextBuffer            *buffer,
+							GossipChatView           *view);
+static gboolean   chat_view_event_cb                   (GossipChatView           *view,
+							GdkEventMotion           *event,
+							GtkTextTag               *tag);
+static gboolean   chat_view_url_event_cb               (GtkTextTag               *tag,
+							GObject                  *object,
+							GdkEvent                 *event,
+							GtkTextIter              *iter,
+							GtkTextBuffer            *buffer);
+static void       chat_view_open_address               (const gchar              *url);
+static void       chat_view_open_address_cb            (GtkMenuItem              *menuitem,
+							const gchar              *url);
+static void       chat_view_copy_address_cb            (GtkMenuItem              *menuitem,
+							const gchar              *url);
+static void       chat_view_realize_cb                 (GossipChatView           *widget,
+							gpointer                  data);
+static void       chat_view_clear_view_cb              (GtkMenuItem              *menuitem,
+							GossipChatView           *view);
+static void       chat_view_insert_text_with_emoticons (GtkTextBuffer            *buf,
+							GtkTextIter              *iter,
+							const gchar              *tag,
+							const gchar              *str);
+static GdkPixbuf *chat_view_get_smiley                 (GossipSmiley              smiley);
+static void       chat_view_maybe_append_timestamp     (GossipChatView           *view,
+							gossip_time_t             timestamp);
+static void       chat_view_maybe_append_datestamp     (GossipChatView           *view);
+static gboolean   chat_view_is_scrolled_down           (GossipChatView           *view);
+static void       chat_view_invite_accept_cb           (GtkWidget                *button,
+							gchar                    *invite);
+static void       chat_view_invite_join_cb             (GossipChatroomProvider   *provider,
+							GossipChatroomJoinResult  result,
+							gint                      id,
+							gpointer                  user_data);
 
 
 extern GConfClient *gconf_client;
 static GObjectClass *parent_class = NULL;
 
-GType
-gossip_chat_view_get_type (void)
-{
-	static GType object_type = 0;
-	
-	if (!object_type) {
-		static const GTypeInfo object_info = {
-			sizeof (GossipChatViewClass),
-			NULL,		/* base_init */
-			NULL,		/* base_finalize */
-			(GClassInitFunc) chat_view_class_init,
-			NULL,		/* class_finalize */
-			NULL,		/* class_data */
-			sizeof (GossipChatView),
-			0,              /* n_preallocs */
-			(GInstanceInitFunc) chat_view_init,
-		};
 
-		object_type = g_type_register_static (GTK_TYPE_TEXT_VIEW,
-                                                      "GossipChatView", 
-                                                      &object_info, 0);
-	}
+G_DEFINE_TYPE (GossipChatView, gossip_chat_view, GTK_TYPE_TEXT_VIEW);
 
-	return object_type;
-}
 
 static void
-chat_view_class_init (GossipChatViewClass *klass)
+gossip_chat_view_class_init (GossipChatViewClass *klass)
 {
 	GObjectClass   *object_class = G_OBJECT_CLASS (klass);
 	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
@@ -182,7 +193,7 @@ chat_view_class_init (GossipChatViewClass *klass)
 }
 
 static void
-chat_view_init (GossipChatView *view)
+gossip_chat_view_init (GossipChatView *view)
 {
 	GossipChatViewPriv *priv;
 
@@ -191,6 +202,11 @@ chat_view_init (GossipChatView *view)
 	
 	priv->buffer = gtk_text_buffer_new (NULL);
 	
+	g_signal_connect (priv->buffer,
+			  "changed",
+			  G_CALLBACK (chat_view_buffer_changed_cb),
+			  view);
+
 	gtk_text_view_set_buffer (GTK_TEXT_VIEW (view), priv->buffer);
 
 	g_object_set (view,
@@ -277,6 +293,11 @@ chat_view_setup_tags (GossipChatView *view)
 				    "justification", GTK_JUSTIFY_CENTER,
 				    NULL);
 
+	gtk_text_buffer_create_tag (priv->buffer,
+				    "invite",
+				    "foreground", "sienna",
+				    NULL);
+
 	tag = gtk_text_buffer_create_tag (priv->buffer,
 					  "url",
 					  "foreground", "steelblue",
@@ -295,8 +316,6 @@ chat_view_setup_tags (GossipChatView *view)
 			  tag);
 }
 
-
-
 static void
 chat_view_populate_popup (GossipChatView *view,
 			  GtkMenu        *menu,
@@ -371,6 +390,37 @@ chat_view_populate_popup (GossipChatView *view,
 	gtk_widget_show (item);
 }
 
+static void
+chat_view_buffer_changed_cb (GtkTextBuffer  *buffer, 
+			     GossipChatView *view)
+{
+	GossipChatViewPriv *priv;
+	GtkTextIter         top, bottom;
+	gint                line;
+	gint                remove;
+	
+	priv = view->priv;
+
+	gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER (priv->buffer), &bottom);
+	line = gtk_text_iter_get_line (&bottom);
+
+	if (line < MAX_LINES) {
+		return;
+	}
+
+	remove = line - MAX_LINES;
+	gtk_text_buffer_get_start_iter (GTK_TEXT_BUFFER (priv->buffer), &top);
+
+	bottom = top;
+	if (!gtk_text_iter_forward_lines (&bottom, remove)) {
+		/* couldn't move iter 'n' lines down */
+		return;
+	}
+
+ 	gtk_text_buffer_delete (GTK_TEXT_BUFFER (priv->buffer),  
+ 				&top, &bottom); 
+}
+
 static gboolean
 chat_view_event_cb (GossipChatView *view,
 		    GdkEventMotion *event,
@@ -486,6 +536,9 @@ chat_view_copy_address_cb (GtkMenuItem *menuitem, const gchar *url)
 
 	clipboard = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
 	gtk_clipboard_set_text (clipboard, url, -1);
+
+	clipboard = gtk_clipboard_get (GDK_SELECTION_PRIMARY);
+	gtk_clipboard_set_text (clipboard, url, -1);
 }
 
 static void
@@ -511,6 +564,7 @@ chat_view_clear_view_cb (GtkMenuItem *menuitem, GossipChatView *view)
 static void
 chat_view_insert_text_with_emoticons (GtkTextBuffer *buf,
 				      GtkTextIter   *iter, 
+				      const gchar   *tag,
 				      const gchar   *str)
 {
 	const gchar *p;
@@ -526,7 +580,13 @@ chat_view_insert_text_with_emoticons (GtkTextBuffer *buf,
 		NULL);
 	
 	if (!use_smileys) {
-		gtk_text_buffer_insert (buf, iter, str, -1);
+		gtk_text_buffer_insert_with_tags_by_name (buf,
+							  iter,
+							  str,
+							  -1,
+							  tag,
+							  NULL);
+/* 		gtk_text_buffer_insert (buf, iter, str, -1); */
 		return;
 	}
 	
@@ -578,14 +638,32 @@ chat_view_insert_text_with_emoticons (GtkTextBuffer *buf,
 
 			if (start > str) {
 				len = start - str;
-				gtk_text_buffer_insert (buf, iter, str, len);
+/* 				gtk_text_buffer_insert (buf, iter, str, len); */
+				gtk_text_buffer_insert_with_tags_by_name (buf,
+									  iter,
+									  str,
+									  len,
+									  tag,
+									  NULL);
 			}
 			
 			pixbuf = chat_view_get_smiley (smileys[match].smiley);
 			gtk_text_buffer_insert_pixbuf (buf, iter, pixbuf);
-			gtk_text_buffer_insert (buf, iter, " ", 1);
+			gtk_text_buffer_insert_with_tags_by_name (buf,
+								  iter,
+								  " ",
+								  -1,
+								  tag,
+								  NULL);
+/* 			gtk_text_buffer_insert (buf, iter, " ", 1); */
 		} else {
-			gtk_text_buffer_insert (buf, iter, str, -1);
+			gtk_text_buffer_insert_with_tags_by_name (buf,
+								  iter,
+								  str,
+								  -1,
+								  tag,
+								  NULL);
+/* 			gtk_text_buffer_insert (buf, iter, str, -1); */
 			return;
 		}
 
@@ -633,27 +711,33 @@ chat_view_get_smiley (GossipSmiley smiley)
 }
 
 static void
-chat_view_maybe_append_timestamp (GossipChatView *view, const gchar *time_str)
+chat_view_maybe_append_timestamp (GossipChatView *view, gossip_time_t timestamp)
 {
 	GossipChatViewPriv *priv;
-	gchar              *stamp;
 	GTimeVal            cur_time;
-
+	
 	g_return_if_fail (GOSSIP_IS_CHAT_VIEW (view));
 
 	priv = view->priv;
 	
-	stamp = gossip_utils_get_timestamp (time_str);
-
 	g_get_current_time (&cur_time);
 
-	if (priv->last_timestamp.tv_sec + TIMESTAMP_INTERVAL < cur_time.tv_sec) {
+	if (priv->last_timestamp.tv_sec + TIMESTAMP_INTERVAL < cur_time.tv_sec){
+		gchar  *stamp;
+		time_t  t;
+		
 		priv->last_timestamp.tv_sec = cur_time.tv_sec;
 
-		gossip_chat_view_append_event_msg (view, stamp, FALSE);
+		if (timestamp < 0) {
+			t = gossip_time_get_current ();
+		} else {
+			t = timestamp;
+		}
+
+		stamp = gossip_time_to_timestamp (t);
+		gossip_chat_view_append_event_message (view, stamp, FALSE);
+		g_free (stamp);
 	} 
-	
-	g_free (stamp);
 }
 
 static void
@@ -675,7 +759,7 @@ chat_view_maybe_append_datestamp (GossipChatView *view)
 
 	g_date_strftime (date_str, 256, _("%A %d %B %Y"), cur_date);
 	
-	gossip_chat_view_append_event_msg (view, date_str, TRUE);
+	gossip_chat_view_append_event_message (view, date_str, TRUE);
 
 	g_get_current_time (&cur_time);
 	priv->last_timestamp.tv_sec = cur_time.tv_sec;
@@ -695,12 +779,6 @@ chat_view_is_scrolled_down (GossipChatView *view)
 {
 	GtkWidget *sw;
 
-	/* Turn off for version before 2.2.3. */
-	if (GTK_MINOR_VERSION < 2 ||
-	    (GTK_MINOR_VERSION == 2 && GTK_MICRO_VERSION < 3)) {
-		return TRUE;
-	}
-
 	sw = gtk_widget_get_parent (GTK_WIDGET (view));
 	if (GTK_IS_SCROLLED_WINDOW (sw)) {
 		GtkAdjustment *vadj;
@@ -788,25 +866,27 @@ finished:
 
 void
 gossip_chat_view_append_chat_message (GossipChatView *view,
-				      const gchar    *time_str,
+				      gossip_time_t   timestamp,
+				      const gchar    *invite,
 				      const gchar    *to,
 				      const gchar    *from,
 				      const gchar    *msg)
 {
 	GtkTextBuffer *buffer;
 	GtkTextIter    iter;
-	gchar         *nick_tag;
+	gchar         *nick_tag, *body_tag = NULL;
 	gint           num_matches, i;
 	GArray        *start, *end;
 	gboolean       bottom;
 
-	if (msg == NULL || msg[0] == 0) {
+	if ((!msg || msg[0] == 0) && 
+	    (!invite || invite[0] == 0)) {
 		return;
 	}
 
 	chat_view_maybe_append_datestamp (view);
-	chat_view_maybe_append_timestamp (view, time_str);
-	
+	chat_view_maybe_append_timestamp (view, timestamp);
+
 	bottom = chat_view_is_scrolled_down (view);
 
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
@@ -878,32 +958,84 @@ gossip_chat_view_append_chat_message (GossipChatView *view,
 	end = g_array_new (FALSE, FALSE, sizeof (gint));
 	
 	num_matches = gossip_utils_url_regex_match (msg, start, end);
-
+	if (invite) {
+		num_matches = 0;
+		body_tag = "invite";
+	}
+		
+		
 	if (num_matches == 0) {
 		gtk_text_buffer_get_end_iter (buffer, &iter);
-		chat_view_insert_text_with_emoticons (buffer, &iter, msg);
+
+		if (invite) {
+			GtkWidget          *widget;
+			GtkTextChildAnchor *anchor;
+			const gchar        *used_msg;
+			const gchar        *used_invite;
+			gchar              *str;
+			
+			used_msg = (msg && strlen (msg) > 0) ? 
+				msg : _("You have been invited to join a chat conference.");
+			
+			/* don't include the invite in the chat window
+			   if it is part of the actual request - some
+			   chat clients send this and it looks weird
+			   repeated */
+			used_invite = (!strstr (msg, invite)) ? 
+				invite : NULL;
+
+			str = g_strdup_printf ("\n%s\n%s%s%s%s--\n",
+					       used_msg,
+					       used_invite ? "(" : "",
+					       used_invite ? used_invite : "",
+					       used_invite ? ")" : "",
+					       used_invite ? "\n" : "");
+
+			chat_view_insert_text_with_emoticons (buffer, &iter, body_tag, str);
+			g_free (str);
+
+			gtk_text_buffer_get_end_iter (buffer, &iter);
+
+			anchor = gtk_text_buffer_create_child_anchor (buffer, &iter);
+			widget = gtk_button_new_with_label (_("Accept"));
+			
+			g_signal_connect (widget, "clicked",
+					  G_CALLBACK (chat_view_invite_accept_cb),
+					  g_strdup (invite));
+
+			gtk_text_view_add_child_at_anchor (GTK_TEXT_VIEW (view),
+							   widget,
+							   anchor);
+			
+			gtk_widget_show_all (widget);
+
+		} else {
+			/* insert text as normal */
+			chat_view_insert_text_with_emoticons (buffer, &iter, body_tag, msg);
+		}
 	} else {
 		gint   last = 0;
 		gint   s = 0, e = 0;
 		gchar *tmp;
-
+		
 		for (i = 0; i < num_matches; i++) {
-
+			
 			s = g_array_index (start, gint, i);
 			e = g_array_index (end, gint, i);
-
+			
 			if (s > last) {
 				tmp = gossip_utils_substring (msg, last, s);
-
+				
 				gtk_text_buffer_get_end_iter (buffer, &iter);
 				chat_view_insert_text_with_emoticons (buffer,
 								      &iter,
+								      NULL,
 								      tmp);
 				g_free (tmp);
 			}
-
+			
 			tmp = gossip_utils_substring (msg, s, e);
-
+			
 			gtk_text_buffer_get_end_iter (buffer, &iter);
 			gtk_text_buffer_insert_with_tags_by_name (buffer,
 								  &iter,
@@ -911,22 +1043,24 @@ gossip_chat_view_append_chat_message (GossipChatView *view,
 								  -1,
 								  "url",
 								  NULL);
-
+			
 			g_free (tmp);
-
+			
 			last = e;
 		}
-
+		
 		if (e < strlen (msg)) {
 			tmp = gossip_utils_substring (msg, e, strlen (msg));
-
+			
 			gtk_text_buffer_get_end_iter (buffer, &iter);
 			chat_view_insert_text_with_emoticons (buffer,
-							      &iter, tmp);
+							      &iter,  
+							      NULL, 
+							      tmp);
 			g_free (tmp);
 		}
 	}
-
+	
 	g_array_free (start, TRUE);
 	g_array_free (end, TRUE);
 	
@@ -945,9 +1079,9 @@ gossip_chat_view_append_chat_message (GossipChatView *view,
 }
 
 void
-gossip_chat_view_append_event_msg (GossipChatView *view, 
-				   const gchar    *str, 
-				   gboolean        timestamp)
+gossip_chat_view_append_event_message (GossipChatView *view, 
+				       const gchar    *str, 
+				       gboolean        timestamp)
 {
 	GtkTextBuffer *buffer;
 	GtkTextIter    iter;
@@ -965,7 +1099,7 @@ gossip_chat_view_append_event_msg (GossipChatView *view,
 	}
 
 	if (timestamp) {
-		stamp = gossip_utils_get_timestamp (NULL);
+		stamp = gossip_time_to_timestamp (-1);
 		msg = g_strdup_printf (" %s - %s", str, stamp);
 		g_free (stamp);
 	} else {
@@ -1047,3 +1181,35 @@ gossip_chat_view_copy_clipboard (GossipChatView *view)
 	gtk_text_buffer_copy_clipboard (buffer, clipboard);
 }
 
+static void
+chat_view_invite_accept_cb (GtkWidget *button, 
+			    gchar     *invite)
+{
+	GossipSession          *session;
+	GossipChatroomProvider *provider;
+	const gchar            *nickname;
+
+	gtk_widget_set_sensitive (button, FALSE);
+	
+	/* FIXME: should really have a session associated with the
+	   chatview */ 
+	session = gossip_app_get_session ();
+	provider = gossip_session_get_chatroom_provider (session);
+	nickname = gossip_session_get_nickname (session);
+
+	gossip_chatroom_provider_invite_accept (provider,
+						chat_view_invite_join_cb,
+						nickname,
+						invite);
+	
+	g_free (invite);
+}
+
+static void
+chat_view_invite_join_cb (GossipChatroomProvider   *provider, 
+			  GossipChatroomJoinResult  result,
+			  gint                      id,
+			  gpointer                  user_data)
+{
+	gossip_group_chat_show (provider, id);
+}
