@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2003      Imendio HB
- * Copyright (C) 2002-2003 Richard Hult <richard@imendio.com>
- * Copyright (C) 2002-2003 Mikael Hallendal <micke@imendio.com>
+ * Copyright (C) 2002-2003 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -25,6 +23,8 @@
 
 #include <gtk/gtktextview.h>
 
+#include <libgossip/gossip-message.h>
+
 #define GOSSIP_TYPE_CHAT_VIEW         (gossip_chat_view_get_type ())
 #define GOSSIP_CHAT_VIEW(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), GOSSIP_TYPE_CHAT_VIEW, GossipChatView))
 #define GOSSIP_CHAT_VIEW_CLASS(k)     (G_TYPE_CHECK_CLASS_CAST((k), GOSSIP_TYPE_CHAT_VIEW, GossipChatViewClass))
@@ -32,32 +32,42 @@
 #define GOSSIP_IS_CHAT_VIEW_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), GOSSIP_TYPE_CHAT_VIEW))
 #define GOSSIP_CHAT_VIEW_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), GOSSIP_TYPE_CHAT_VIEW, GossipChatViewClass))
 
+
 typedef struct _GossipChatView      GossipChatView;
 typedef struct _GossipChatViewClass GossipChatViewClass;
 typedef struct _GossipChatViewPriv  GossipChatViewPriv;
 
+
 struct _GossipChatView {
 	GtkTextView         parent;
 
 	GossipChatViewPriv *priv;
 };
 
+
 struct _GossipChatViewClass {
 	GtkTextViewClass    parent_class;
 };
 
-GType            gossip_chat_view_get_type            (void) G_GNUC_CONST;
-GossipChatView * gossip_chat_view_new                 (void);
-void             gossip_chat_view_append_chat_message (GossipChatView *view,
-						       const gchar    *time_str,
-						       const gchar    *to,
-						       const gchar    *from,
-						       const gchar    *msg);
-void             gossip_chat_view_append_event_msg    (GossipChatView *view,
-						       const gchar    *str,
-						       gboolean        timestamp);
-void             gossip_chat_view_set_margin          (GossipChatView *view,
-						       gint            margin);
-void             gossip_chat_view_clear               (GossipChatView *view);
+
+GType            gossip_chat_view_get_type             (void) G_GNUC_CONST;
+GossipChatView * gossip_chat_view_new                  (void);
+void             gossip_chat_view_append_chat_message  (GossipChatView *view,
+							gossip_time_t   timestamp,
+							const gchar    *invite,
+							const gchar    *to,
+							const gchar    *from,
+							const gchar    *msg);
+void             gossip_chat_view_append_event_message (GossipChatView *view,
+							const gchar    *str,
+							gboolean        timestamp);
+void             gossip_chat_view_set_margin           (GossipChatView *view,
+							gint            margin);
+void             gossip_chat_view_clear                (GossipChatView *view);
+void             gossip_chat_view_scroll_down          (GossipChatView *view);
+gboolean         gossip_chat_view_get_selection_bounds (GossipChatView *view,
+							GtkTextIter    *start,
+							GtkTextIter    *end);
+void             gossip_chat_view_copy_clipboard       (GossipChatView *view);
 
 #endif /* __GOSSIP_CHAT_VIEW_H__ */
