@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2003-2004 Imendio HB
- * Copyright (C) 2003-2004 Geert-Jan Van den Bogaerde <gvdbogaerde@pandora.be>
+ * Copyright (C) 2003-2004 Imendio AB
+ * Copyright (C) 2003-2004 Geert-Jan Van den Bogaerde <geertjan@gnome.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,116 +24,114 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include <glade/glade.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <gconf/gconf-client.h>
+
+#include <libgossip/gossip-protocol.h>
+
 #include "gossip-app.h"
 #include "gossip-contact-info.h"
 #include "gossip-preferences.h"
+#include "gossip-private-chat.h"
 #include "gossip-chat-window.h"
 #include "gossip-notebook.h"
-#include "gossip-roster.h"
-#include "gossip-utils.h"
 #include "gossip-stock.h"
 #include "gossip-sound.h"
 #include "gossip-log.h"
+#include "gossip-ui-utils.h"
 #include "eel-ellipsizing-label.h"
 
-static void gossip_chat_window_class_init     (GossipChatWindowClass *klass);
-static void gossip_chat_window_init	      (GossipChatWindow      *window);
-static void gossip_chat_window_finalize       (GObject		     *object);
-static GdkPixbuf * 
-chat_window_get_status_pixbuf		      (GossipChatWindow	     *window,
-					       GossipChat	     *chat);
-static void chat_window_accel_cb              (GtkAccelGroup         *accelgroup,
-					       GObject               *object,
-					       guint                  key,
-					       GdkModifierType        mod,
-					       GossipChatWindow      *window);
-static gboolean 
-chat_window_has_toplevel_focus                (GossipChatWindow	     *window);
-static GtkWidget*
-chat_window_create_label		      (GossipChatWindow	     *window,
-					       GossipChat	     *chat);
-static void chat_window_update_status	      (GossipChatWindow	     *window,
-					       GossipChat	     *chat);
-static void chat_window_update_title	      (GossipChatWindow	     *window);
-static void chat_window_update_menu	      (GossipChatWindow	     *window);
-static void chat_window_clear_activate_cb     (GtkWidget             *menuitem,
-					       GossipChatWindow      *window);
-static void chat_window_info_activate_cb      (GtkWidget	     *menuitem,
-					       GossipChatWindow	     *window);
-static void chat_window_log_activate_cb       (GtkWidget             *menuitem,
-					       GossipChatWindow      *window);
-static void chat_window_conv_activate_cb      (GtkWidget             *menuitem,
-					       GossipChatWindow      *window);
-static void chat_window_close_activate_cb     (GtkWidget	     *menuitem,
-					       GossipChatWindow      *window);
-static void chat_window_cut_activate_cb       (GtkWidget             *menuitem,
-					       GossipChatWindow      *window);
-static void chat_window_copy_activate_cb      (GtkWidget             *menuitem,
-					       GossipChatWindow      *window);
-static void chat_window_paste_activate_cb     (GtkWidget             *menuitem,
-					       GossipChatWindow      *window);
-static void chat_window_tab_left_activate_cb  (GtkWidget	     *menuitem,
-					       GossipChatWindow      *window);
-static void chat_window_tab_right_activate_cb (GtkWidget	     *menuitem,
-					       GossipChatWindow	     *window);
-static void chat_window_detach_activate_cb    (GtkWidget	     *menuitem,
-					       GossipChatWindow      *window);
-static gboolean chat_window_delete_event_cb   (GtkWidget	     *dialog,
-					       GdkEvent		     *event,
-					       GossipChatWindow	     *window);
-static void chat_window_presence_updated_cb   (gpointer               not_used,
-					       GossipContact         *contact,
-					       GossipChat	     *chat);
-static void chat_window_update_tooltip        (GossipChatWindow      *window,
-					       GossipChat            *chat);
-static void chat_window_name_changed_cb       (GossipChat            *chat,
-					       const gchar           *name,
-					       GossipChatWindow      *window);
-static void chat_window_composing_cb	      (GossipChat	     *chat,
-					       gboolean		      is_composing,
-					       GossipChatWindow	     *window);
-static void chat_window_new_message_cb	      (GossipChat	     *chat,
-					       GossipChatWindow	     *window);
-static void chat_window_disconnected_cb	      (GossipApp	     *app,
-					       GossipChatWindow	     *window);
-static void chat_window_switch_page_cb	      (GtkNotebook	     *notebook,
-					       GtkNotebookPage	     *page,
-					       gint		      page_num,
-					       GossipChatWindow      *window);
-static void chat_window_tab_added_cb	      (GossipNotebook        *notebook,
-					       GtkWidget	     *child,
-					       GossipChatWindow      *window);
-static void chat_window_tab_removed_cb	      (GossipNotebook	     *notebook,
-					       GtkWidget	     *child,
-					       GossipChatWindow      *window);
-static void chat_window_tab_detached_cb       (GossipNotebook        *notebook,
-					       GtkWidget	     *widget,
-					       GossipChatWindow      *window);
-static void chat_window_tabs_reordered_cb     (GossipNotebook	     *notebook,
-					       GossipChatWindow      *window);
-static void chat_window_close_clicked_cb      (GtkWidget	     *button,
-					       GossipChat	     *chat);
-static gboolean
-chat_window_focus_in_event_cb                 (GtkWidget	     *widget,
-					       GdkEvent		     *event,
-					       GossipChatWindow      *window); 
-static const gchar *
-chat_window_get_name                          (GossipChatWindow      *window, 
-					       GossipChat            *chat);
-static void    chat_window_drag_data_received (GtkWidget             *widget,
-					       GdkDragContext        *context,
-					       int                    x,
-					       int                    y, 
-					       GtkSelectionData      *selection,
-					       guint                  info, 
-					       guint                  time, 
-					       GossipChatWindow      *window);
 
-/* Called from Glade, so it shouldn't be static */
+static void       gossip_chat_window_class_init        (GossipChatWindowClass *klass);
+static void       gossip_chat_window_init              (GossipChatWindow      *window);
+static void       gossip_chat_window_finalize          (GObject               *object);
+static GdkPixbuf *chat_window_get_status_pixbuf        (GossipChatWindow      *window,
+							GossipChat            *chat);
+static void       chat_window_accel_cb                 (GtkAccelGroup         *accelgroup,
+							GObject               *object,
+							guint                  key,
+							GdkModifierType        mod,
+							GossipChatWindow      *window);
+static GtkWidget *chat_window_create_label             (GossipChatWindow      *window,
+							GossipChat            *chat);
+static void       chat_window_update_status            (GossipChatWindow      *window,
+							GossipChat            *chat);
+static void       chat_window_update_title             (GossipChatWindow      *window);
+static void       chat_window_update_menu              (GossipChatWindow      *window);
+static void       chat_window_invite_menu_setup        (GossipChatWindow      *window);
+static void       chat_window_clear_activate_cb        (GtkWidget             *menuitem,
+							GossipChatWindow      *window);
+static void       chat_window_info_activate_cb         (GtkWidget             *menuitem,
+							GossipChatWindow      *window);
+static void       chat_window_log_activate_cb          (GtkWidget             *menuitem,
+							GossipChatWindow      *window);
+static void       chat_window_conv_activate_cb         (GtkWidget             *menuitem,
+							GossipChatWindow      *window);
+static void       chat_window_show_contacts_toggled_cb (GtkWidget             *menuitem,
+							GossipChatWindow      *window);
+static void       chat_window_close_activate_cb        (GtkWidget             *menuitem,
+							GossipChatWindow      *window);
+static void       chat_window_cut_activate_cb          (GtkWidget             *menuitem,
+							GossipChatWindow      *window);
+static void       chat_window_copy_activate_cb         (GtkWidget             *menuitem,
+							GossipChatWindow      *window);
+static void       chat_window_paste_activate_cb        (GtkWidget             *menuitem,
+							GossipChatWindow      *window);
+static void       chat_window_tab_left_activate_cb     (GtkWidget             *menuitem,
+							GossipChatWindow      *window);
+static void       chat_window_tab_right_activate_cb    (GtkWidget             *menuitem,
+							GossipChatWindow      *window);
+static void       chat_window_detach_activate_cb       (GtkWidget             *menuitem,
+							GossipChatWindow      *window);
+static gboolean   chat_window_delete_event_cb          (GtkWidget             *dialog,
+							GdkEvent              *event,
+							GossipChatWindow      *window);
+static void       chat_window_status_changed_cb        (GossipChat            *chat,
+							GossipChatWindow      *window);
+static void       chat_window_update_tooltip           (GossipChatWindow      *window,
+							GossipChat            *chat);
+static void       chat_window_name_changed_cb          (GossipChat            *chat,
+							const gchar           *name,
+							GossipChatWindow      *window);
+static void       chat_window_composing_cb             (GossipChat            *chat,
+							gboolean               is_composing,
+							GossipChatWindow      *window);
+static void       chat_window_new_message_cb           (GossipChat            *chat,
+							GossipChatWindow      *window);
+static void       chat_window_disconnected_cb          (GossipApp             *app,
+							GossipChatWindow      *window);
+static void       chat_window_switch_page_cb           (GtkNotebook           *notebook,
+							GtkNotebookPage       *page,
+							gint                   page_num,
+							GossipChatWindow      *window);
+static void       chat_window_tab_added_cb             (GossipNotebook        *notebook,
+							GtkWidget             *child,
+							GossipChatWindow      *window);
+static void       chat_window_tab_removed_cb           (GossipNotebook        *notebook,
+							GtkWidget             *child,
+							GossipChatWindow      *window);
+static void       chat_window_tab_detached_cb          (GossipNotebook        *notebook,
+							GtkWidget             *widget,
+							GossipChatWindow      *window);
+static void       chat_window_tabs_reordered_cb        (GossipNotebook        *notebook,
+							GossipChatWindow      *window);
+static gboolean   chat_window_focus_in_event_cb        (GtkWidget             *widget,
+							GdkEvent              *event,
+							GossipChatWindow      *window);
+static void       chat_window_drag_data_received       (GtkWidget             *widget,
+							GdkDragContext        *context,
+							int                    x,
+							int                    y,
+							GtkSelectionData      *selection,
+							guint                  info,
+							guint                  time,
+							GossipChatWindow      *window);
+
+
+/* called from Glade, so it shouldn't be static */
 GtkWidget * chat_window_create_notebook (gpointer data);
 
+
 struct _GossipChatWindowPriv {
 	GList       *chats;
 	GList       *chats_new_msg;
@@ -150,6 +148,10 @@ struct _GossipChatWindowPriv {
 	GtkWidget   *m_conv_clear;
 	GtkWidget   *m_conv_log;
 	GtkWidget   *m_conv_info;
+	GtkWidget   *s_conv_show_contacts;
+	GtkWidget   *m_conv_show_contacts;
+	GtkWidget   *s_conv_invite_contacts;
+	GtkWidget   *m_conv_invite_contacts;
 	GtkWidget   *m_conv_close;
 	GtkWidget   *m_edit_cut;
 	GtkWidget   *m_edit_copy;
@@ -161,48 +163,29 @@ struct _GossipChatWindowPriv {
 	GtkWidget   *m_tabs_detach;
 };
 
-static gpointer parent_class;
-static GList *chat_windows = NULL;
+
+static gpointer  parent_class;
+static GList    *chat_windows = NULL;
+
 
 static guint tab_accel_keys[] = {
 	GDK_1, GDK_2, GDK_3, GDK_4, GDK_5,
 	GDK_6, GDK_7, GDK_8, GDK_9, GDK_0
 };
 
+
 enum DndDropType {
-	DND_DROP_TYPE_JID,
-	NUM_DROP_TYPES
+	DND_DROP_TYPE_CONTACT_ID,
 };
 
+
 static GtkTargetEntry drop_types[] = {
-	{ "text/jid", 0, DND_DROP_TYPE_JID },
+	{ "text/contact-id", 0, DND_DROP_TYPE_CONTACT_ID },
 };
 
-GType
-gossip_chat_window_get_type (void)
-{
-        static GType type_id = 0;
 
-        if (type_id == 0) {
-                static const GTypeInfo type_info = {
-                        sizeof (GossipChatWindowClass),
-                        NULL,
-                        NULL,
-                        (GClassInitFunc) gossip_chat_window_class_init,
-                        NULL,
-                        NULL,
-                        sizeof (GossipChatWindow),
-                        0,
-                        (GInstanceInitFunc) gossip_chat_window_init
-                };
+G_DEFINE_TYPE (GossipChatWindow, gossip_chat_window, G_TYPE_OBJECT);
 
-                type_id = g_type_register_static (G_TYPE_OBJECT,
-                                                  "GossipChatWindow",
-                                                  &type_info, 0);
-        }
-
-        return type_id;
-}
 
 static void
 gossip_chat_window_class_init (GossipChatWindowClass *klass)
@@ -235,6 +218,10 @@ gossip_chat_window_init (GossipChatWindow *window)
 				       "menu_conv_clear", &priv->m_conv_clear,
 				       "menu_conv_info", &priv->m_conv_info,
 				       "menu_conv_log", &priv->m_conv_log,
+				       "sep_conv_show_contacts", &priv->s_conv_show_contacts,
+				       "menu_conv_show_contacts", &priv->m_conv_show_contacts,
+				       "sep_conv_invite_contacts", &priv->s_conv_invite_contacts,
+				       "menu_conv_invite_contacts", &priv->m_conv_invite_contacts,
 				       "menu_conv_close", &priv->m_conv_close,
 				       "menu_edit_cut", &priv->m_edit_cut,
 				       "menu_edit_copy", &priv->m_edit_copy,
@@ -279,6 +266,10 @@ gossip_chat_window_init (GossipChatWindow *window)
 			  "activate",
 			  G_CALLBACK (chat_window_info_activate_cb),
 			  window);
+	g_signal_connect (priv->m_conv_show_contacts,
+			  "toggled",
+			  G_CALLBACK (chat_window_show_contacts_toggled_cb),
+			  window);
 	g_signal_connect (priv->m_conv_close,
 			  "activate",
 			  G_CALLBACK (chat_window_close_activate_cb),
@@ -343,16 +334,19 @@ gossip_chat_window_init (GossipChatWindow *window)
 			  "tabs_reordered",
 			  G_CALLBACK (chat_window_tabs_reordered_cb),
 			  window);
-	g_signal_connect (gossip_app_get (),
+	g_signal_connect (gossip_app_get_session (),
 			  "disconnected",
 			  G_CALLBACK (chat_window_disconnected_cb),
 			  window);
 
-	gtk_drag_dest_set (GTK_WIDGET (priv->notebook), GTK_DEST_DEFAULT_ALL,
-			   drop_types, NUM_DROP_TYPES,
+	gtk_drag_dest_set (GTK_WIDGET (priv->notebook), 
+			   GTK_DEST_DEFAULT_ALL,
+			   drop_types, 
+			   G_N_ELEMENTS (drop_types),
 			   GDK_ACTION_COPY);
 
-	g_signal_connect (priv->notebook, "drag-data-received",
+	g_signal_connect (priv->notebook, 
+			  "drag-data-received",
 			  G_CALLBACK (chat_window_drag_data_received),
 			  window);
 
@@ -364,12 +358,80 @@ gossip_chat_window_init (GossipChatWindow *window)
 	priv->current_chat = NULL;
 }
 
+/* Returns the window to open a new tab in if there is only one window visble */
+/* Otherwise, returns NULL indicating that a new window should be added       */
+GossipChatWindow *
+gossip_chat_window_get_default (gboolean for_group_chats)
+{
+	GossipChatWindow *default_chat_window = NULL;
+	GList            *l;
+
+	for (l=chat_windows; l; l=l->next) {
+		GossipChatWindow *chat_window;
+		GtkWidget        *dialog;
+		GdkWindow        *window;
+		gboolean          visible;
+		gint              count;
+
+		chat_window = l->data;
+
+		count = gossip_chat_window_get_group_chats (chat_window);
+		if ((count < 1 && for_group_chats) || (count > 0 && !for_group_chats)) {
+			continue;
+		}
+
+		dialog = gossip_chat_window_get_dialog (chat_window);
+		window = dialog->window;
+
+		g_object_get (dialog, 
+			      "visible", &visible,
+			      NULL);
+		
+		visible = visible && !(gdk_window_get_state (window) & GDK_WINDOW_STATE_ICONIFIED);
+
+		/* more than one visible window */
+		if (visible && default_chat_window) {
+			return NULL;
+		}
+
+		if (visible) {
+			default_chat_window = chat_window;
+		}
+	}
+
+	return default_chat_window;
+}
+
+gint 
+gossip_chat_window_get_group_chats (GossipChatWindow *window)
+{
+	GossipChatWindowPriv *priv;
+ 	GList                *l;
+	gint                  count;
+
+	g_return_val_if_fail (window != NULL, 0);
+
+	priv = window->priv;
+	
+	for (l=priv->chats, count=0; l; l=l->next) {
+		GossipChat *chat;
+
+		chat = GOSSIP_CHAT (l->data);
+
+		if (gossip_chat_get_group_chat (chat)) {
+			count++;
+		}
+	}
+
+	return count;
+}
+
 static void
 gossip_chat_window_finalize (GObject *object)
 {
 	GossipChatWindow *window = GOSSIP_CHAT_WINDOW (object);
 
-	g_signal_handlers_disconnect_by_func (gossip_app_get (),
+	g_signal_handlers_disconnect_by_func (gossip_app_get_session (),
 					      chat_window_disconnected_cb,
 					      window);
 
@@ -384,25 +446,27 @@ static GdkPixbuf *
 chat_window_get_status_pixbuf (GossipChatWindow *window,
 			       GossipChat *chat)
 {
-	GdkPixbuf   *pixbuf;
+	GossipSession *session;
+	GdkPixbuf     *pixbuf;
+
+	session = gossip_app_get_session ();
 
 	if (g_list_find (window->priv->chats_new_msg, chat)) {
-		pixbuf = gossip_utils_get_pixbuf_from_stock (GOSSIP_STOCK_MESSAGE);
+		pixbuf = gossip_ui_utils_get_pixbuf_from_stock (GOSSIP_STOCK_MESSAGE);
 	}
 	else if (g_list_find (window->priv->chats_composing, chat)) {
-		pixbuf = gossip_utils_get_pixbuf_from_stock (GOSSIP_STOCK_TYPING);
+		pixbuf = gossip_ui_utils_get_pixbuf_from_stock (GOSSIP_STOCK_TYPING);
 	}
-	else if (!gossip_app_is_connected ()) {
-		/* Always show offline if we're not connected */
-		pixbuf = gossip_utils_get_pixbuf_offline ();
+	else if (!gossip_session_is_connected (session)) {
+		/* FIXME: should use protocol for contact since the
+		   protocol we use to send the image may not be online
+		   and a session may cover multiple accounts */
+
+		/* always offline if disconnected */
+		pixbuf = gossip_ui_utils_get_pixbuf_offline ();
 	}
 	else {
-		GossipContact  *contact;
-		GossipPresence *presence;
-		
-		contact = gossip_chat_get_contact (chat);
-		presence = gossip_contact_get_presence (contact);
-		pixbuf = gossip_presence_get_pixbuf (presence);
+		pixbuf = gossip_chat_get_status_pixbuf (chat);
 	}
 
 	return pixbuf;
@@ -430,86 +494,57 @@ chat_window_accel_cb (GtkAccelGroup    *accelgroup,
 	}
 }
 
-static gboolean
-chat_window_has_toplevel_focus (GossipChatWindow *window)
-{
-	gboolean focus = FALSE;
-	
-	/* The has-toplevel-focus property is new in GTK 2.2 so if we don't find it, we
-	 * pretend that the window doesn't have focus.
-	 */
-	if (g_object_class_find_property (G_OBJECT_GET_CLASS (window->priv->dialog),
-					  "has-toplevel-focus")) {
-		g_object_get (window->priv->dialog, "has-toplevel-focus", &focus, NULL);
-	}
-
-	return focus;
-}
-
 static GtkWidget *
 chat_window_create_label (GossipChatWindow *window,
 			  GossipChat       *chat)
 {
 	GossipChatWindowPriv *priv;
 	GtkWidget            *hbox;
-	GtkWidget            *close_button;
-	GtkWidget            *close_img;
 	GtkWidget            *name_label;
 	GtkWidget            *status_img;
 	const gchar          *name;
-	gint	              w, h;
-	GtkWidget            *evbox_tooltip;
-	GtkWidget            *evbox_hbox;
-	
+	GtkWidget            *event_box; 
+	PangoAttrList        *attr_list;
+	PangoAttribute       *attr;
+
 	priv = window->priv;
 	
-	hbox = gtk_hbox_new (FALSE, 0);
+	hbox = gtk_hbox_new (FALSE, 2);
 
-	evbox_tooltip = gtk_event_box_new ();
-
-	/* Make the event box invisible. */
-	gtk_event_box_set_visible_window (GTK_EVENT_BOX (evbox_tooltip), FALSE);
-	
-	status_img = gtk_image_new ();
-	g_object_set_data (G_OBJECT (chat), "chat-window-status-img", status_img);
-	g_object_set_data (G_OBJECT (chat), "chat-window-tooltip-widget",
-			   evbox_tooltip);
-
-	name = chat_window_get_name (window, chat);
+ 	event_box = gtk_event_box_new (); 
+ 	gtk_event_box_set_visible_window (GTK_EVENT_BOX (event_box), FALSE); 
 
+	name = gossip_chat_get_name (chat);
 	name_label = gtk_label_new (name);
 
-	chat_window_update_tooltip (window, chat);
+	gtk_label_set_ellipsize (GTK_LABEL (name_label), PANGO_ELLIPSIZE_END);
 	
-	gtk_misc_set_padding (GTK_MISC (name_label), 2, 0);
+	attr_list = pango_attr_list_new ();
+	attr = pango_attr_scale_new (1/1.2);
+	attr->start_index = 0;
+	attr->end_index = -1;
+	pango_attr_list_insert (attr_list, attr);
+	gtk_label_set_attributes (GTK_LABEL (name_label), attr_list);
+	pango_attr_list_unref (attr_list);
+
 	gtk_misc_set_alignment (GTK_MISC (name_label), 0.0, 0.5);
-
 	g_object_set_data (G_OBJECT (chat), "label", name_label); 
-		
-	gtk_icon_size_lookup (GTK_ICON_SIZE_MENU, &w, &h);
-	close_button = gtk_button_new ();
-	gtk_button_set_relief (GTK_BUTTON (close_button), GTK_RELIEF_NONE);
-	close_img = gtk_image_new_from_stock (GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
-	gtk_widget_set_size_request (close_button, w, h);
-	gtk_container_add (GTK_CONTAINER (close_button), close_img);
 
-	evbox_hbox = gtk_hbox_new (FALSE, 0);
+	status_img = gtk_image_new ();
 
-	gtk_box_pack_start (GTK_BOX (evbox_hbox), status_img, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (evbox_hbox), name_label, TRUE, TRUE, 0);
+	g_object_set_data (G_OBJECT (chat), "chat-window-status-img", status_img);
+  	g_object_set_data (G_OBJECT (chat), "chat-window-tooltip-widget", event_box);
 
-	gtk_container_add (GTK_CONTAINER (evbox_tooltip), evbox_hbox);
-	gtk_box_pack_start (GTK_BOX (hbox), evbox_tooltip, TRUE, TRUE, 0);
-	gtk_box_pack_end (GTK_BOX (hbox), close_button, FALSE, FALSE, 0);
+	chat_window_update_tooltip (window, chat);
 
-	g_signal_connect (close_button,
-			  "clicked",
-			  G_CALLBACK (chat_window_close_clicked_cb),
-			  chat);
+	gtk_box_pack_start (GTK_BOX (hbox), status_img, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), name_label, TRUE, TRUE, 0);
 
-	gtk_widget_show_all (hbox);
+ 	gtk_container_add (GTK_CONTAINER (event_box), hbox);
 
-	return hbox;
+ 	gtk_widget_show_all (event_box);
+
+	return event_box;
 }
 
 static void
@@ -517,11 +552,11 @@ chat_window_update_status (GossipChatWindow *window, GossipChat *chat)
 {
 	GtkImage  *image;
 	GdkPixbuf *pixbuf;
-	
+
 	pixbuf = chat_window_get_status_pixbuf (window, chat);
 	image = g_object_get_data (G_OBJECT (chat), "chat-window-status-img");
 	gtk_image_set_from_pixbuf (image, pixbuf);
-
+	
 	chat_window_update_tooltip (window, chat);
 }
 
@@ -536,12 +571,11 @@ chat_window_update_title (GossipChatWindow *window)
 
 	title = g_strdup_printf (_("%sChat - %s"), 
 				 window->priv->new_msg ? "*" : "", 
-				 chat_window_get_name (window, 
-						       priv->current_chat));
+				 gossip_chat_get_name (priv->current_chat));
 
 	gtk_window_set_title (GTK_WINDOW (window->priv->dialog), title);
 	if (window->priv->new_msg) {
-		pixbuf = gossip_utils_get_pixbuf_from_stock (GOSSIP_STOCK_MESSAGE);
+		pixbuf = gossip_ui_utils_get_pixbuf_from_stock (GOSSIP_STOCK_MESSAGE);
 		gtk_window_set_icon (GTK_WINDOW (window->priv->dialog), pixbuf);
         } else {
 		gtk_window_set_icon (GTK_WINDOW (window->priv->dialog), NULL);
@@ -553,11 +587,16 @@ static void
 chat_window_update_menu (GossipChatWindow *window)
 {
 	GossipChatWindowPriv *priv;
+	GossipContact        *contact;
 	gboolean              first_page;
 	gboolean              last_page;
 	gint                  num_pages;
 	gint                  page_num;
 	
+	/* group chat menu details */
+	gboolean              is_group_chat;
+	gboolean              show_contacts;
+
 	priv = window->priv;
 
 	page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (priv->notebook));
@@ -565,11 +604,133 @@ chat_window_update_menu (GossipChatWindow *window)
 	first_page = (page_num == 0);
 	last_page = (page_num == (num_pages - 1));
 
+	contact = gossip_chat_get_contact (priv->current_chat);
+
 	gtk_widget_set_sensitive (priv->m_tabs_next, !last_page);
 	gtk_widget_set_sensitive (priv->m_tabs_prev, !first_page);
 	gtk_widget_set_sensitive (priv->m_tabs_detach, num_pages > 1);
 	gtk_widget_set_sensitive (priv->m_tabs_left, !first_page);
 	gtk_widget_set_sensitive (priv->m_tabs_right, !last_page);
+	gtk_widget_set_sensitive (priv->m_conv_info, contact != NULL);
+
+	is_group_chat = gossip_chat_get_group_chat (priv->current_chat);
+	if (is_group_chat) {
+		gtk_widget_show (priv->s_conv_show_contacts);
+		gtk_widget_show (priv->m_conv_show_contacts);
+
+		show_contacts = gossip_chat_get_show_contacts (priv->current_chat);
+
+		/* we need to block the signal here because all we are
+		   really trying to do is check or uncheck the menu
+		   item.
+
+		   if we don't do this we get funny behaviour with 2
+		   or more group chat windows where showing contacts
+		   doesn't do anything. */
+
+		g_signal_handlers_block_by_func (priv->m_conv_show_contacts, 
+						 chat_window_show_contacts_toggled_cb, 
+						 window);
+		
+		g_object_set (priv->m_conv_show_contacts, 
+			      "active", show_contacts,
+			      NULL);
+
+		g_signal_handlers_unblock_by_func (priv->m_conv_show_contacts, 
+						   chat_window_show_contacts_toggled_cb,
+						   window);
+	} else {
+		gtk_widget_hide (priv->s_conv_show_contacts);
+		gtk_widget_hide (priv->m_conv_show_contacts);
+	}
+
+	chat_window_invite_menu_setup (window); 
+}
+
+static void
+chat_window_invite_menu_setup (GossipChatWindow *window)
+{
+	GossipChatWindowPriv   *priv;
+	GossipSession          *session;
+	GossipChatroomProvider *provider;
+	GList                  *list = NULL;
+
+	gboolean                is_group_chat;
+
+	priv = window->priv;
+
+	session = gossip_app_get_session ();
+	provider = gossip_session_get_chatroom_provider (session);
+
+	is_group_chat = gossip_chat_get_group_chat (priv->current_chat);
+
+	if (is_group_chat) {
+		GossipProtocol *protocol = NULL;
+		const GList    *contacts;
+		const GList    *l;
+
+		contacts = gossip_protocol_get_contacts (protocol);
+		for (l = contacts; l; l = l->next) {
+			GossipContact *contact;
+
+			contact = l->data;
+
+			if (!gossip_contact_is_online (contact)) {
+				continue;
+			}
+
+			list = g_list_append (list, contact);
+		}
+	} else {
+		list = gossip_chatroom_provider_get_rooms (provider);
+	}
+
+	if (0) {
+/* 	if (g_list_length (list) > 0) { */
+		GtkWidget *rooms_menu;
+		GList     *l;
+
+		gtk_widget_show (priv->s_conv_invite_contacts);
+		gtk_widget_show (priv->m_conv_invite_contacts);
+
+		rooms_menu = gtk_menu_new ();
+
+		for (l = list; l; l = l->next) {
+			GossipChatroomId  id;
+			GossipContact    *contact;
+			const gchar      *name;
+			GtkWidget        *item;
+
+			if (is_group_chat) {
+				contact = l->data;
+				name = gossip_contact_get_name (contact);
+			} else {
+				id = GPOINTER_TO_INT(l->data);
+				name = gossip_chatroom_provider_get_room_name (provider, id);
+			}
+
+			if (!name) {
+				continue;
+			}
+
+			item = gtk_menu_item_new_with_label (name);
+			gtk_menu_shell_append (GTK_MENU_SHELL (rooms_menu), item);
+			
+			gtk_widget_show_all (rooms_menu);
+			
+			gtk_menu_item_set_submenu (GTK_MENU_ITEM (priv->m_conv_invite_contacts),
+						   rooms_menu);
+		}
+	} else {
+		gtk_widget_hide (priv->s_conv_invite_contacts);
+		gtk_widget_hide (priv->m_conv_invite_contacts);
+
+		gtk_menu_item_remove_submenu (GTK_MENU_ITEM (priv->m_conv_invite_contacts));
+	}
+
+	if (is_group_chat) {
+		g_list_free (list);
+	}
 }
 
 static void
@@ -581,14 +742,14 @@ chat_window_clear_activate_cb (GtkWidget *menuitem, GossipChatWindow *window)
 }
 
 static void
-chat_window_log_activate_cb (GtkWidget *menuitem,GossipChatWindow *window)
+chat_window_log_activate_cb (GtkWidget *menuitem, GossipChatWindow *window)
 {
 	GossipChatWindowPriv *priv = window->priv;
 	GossipContact        *contact;
 
 	contact = gossip_chat_get_contact (priv->current_chat);
 	
-	gossip_log_show (gossip_contact_get_jid (contact));
+	gossip_log_show (contact);
 }
 
 static void
@@ -601,8 +762,7 @@ chat_window_info_activate_cb (GtkWidget *menuitem, GossipChatWindow *window)
 
 	contact = gossip_chat_get_contact (priv->current_chat);
 
-	gossip_contact_info_new (gossip_contact_get_jid (contact),
-				 chat_window_get_name (window, priv->current_chat));
+	gossip_contact_info_new (contact);
 }
 
 static void
@@ -616,10 +776,31 @@ chat_window_conv_activate_cb (GtkWidget        *menuitem,
 	priv = window->priv;
 	
 	contact = gossip_chat_get_contact (priv->current_chat);
-	log_exists = gossip_log_exists (gossip_contact_get_jid (contact));
+
+	if (contact != NULL) {
+		log_exists = gossip_log_exists (contact);
+	} else {
+		log_exists = FALSE;
+	}
+
 	gtk_widget_set_sensitive (priv->m_conv_log, log_exists);
 }
 
+static void
+chat_window_show_contacts_toggled_cb (GtkWidget        *menuitem,
+				      GossipChatWindow *window)
+{
+	GossipChatWindowPriv *priv;
+	gboolean              show;
+
+	priv = window->priv;
+
+	g_return_if_fail (priv->current_chat != NULL);
+	
+	show = gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (priv->m_conv_show_contacts)); 
+	gossip_chat_set_show_contacts (priv->current_chat, show);
+}
+
 static void
 chat_window_close_activate_cb (GtkWidget        *menuitem,
 			       GossipChatWindow *window)
@@ -751,14 +932,9 @@ chat_window_delete_event_cb (GtkWidget        *dialog,
 }
 
 static void
-chat_window_presence_updated_cb (gpointer           not_used,
-				 GossipContact     *contact,
-				 GossipChat        *chat)
-{ 
-	GossipChatWindow *window;
-
-	window = gossip_chat_get_window (chat);
-
+chat_window_status_changed_cb (GossipChat       *chat,
+		               GossipChatWindow *window)
+{
 	chat_window_update_status (window, chat);
 }
 
@@ -767,36 +943,15 @@ chat_window_update_tooltip (GossipChatWindow *window,
 			    GossipChat       *chat)
 {
 	GossipChatWindowPriv *priv;
-	GossipContact        *contact;
-	GossipJID            *jid;
-	gchar                *str;
-	const gchar          *status;
 	GtkWidget            *widget;
-	GossipPresence       *presence;
+	gchar                *str;
 
 	priv = window->priv;
 
-	contact = gossip_chat_get_contact (chat);
-	jid = gossip_contact_get_jid (contact);
+	widget = g_object_get_data (G_OBJECT (chat), 
+			            "chat-window-tooltip-widget");
 
-	presence = gossip_contact_get_presence (contact);
-	status = gossip_presence_get_status (presence);
-
-	if (!status || strcmp (status, "") == 0) {
-
-		if (gossip_contact_is_online (contact)) {
-			GossipPresenceType p_type;
-
-			p_type = gossip_presence_get_type (presence);
-			status = gossip_utils_get_default_status (p_type);
-		} else {
-			status = _("Offline");
-		}
-	}
-
-	str = g_strdup_printf ("%s\n%s",
-			       gossip_jid_get_without_resource (jid),
-			       status);
+	str = gossip_chat_get_tooltip (chat);
 
 	if (g_list_find (priv->chats_composing, chat)) {
 		gchar *t_str;
@@ -806,9 +961,6 @@ chat_window_update_tooltip (GossipChatWindow *window,
 		g_free (t_str);
 	}
 
-	widget = g_object_get_data (G_OBJECT (chat), 
-				    "chat-window-tooltip-widget");
-
 	gtk_tooltips_set_tip (priv->tooltips,
 			      widget,
 			      str,
@@ -847,7 +999,7 @@ static void
 chat_window_new_message_cb (GossipChat       *chat,
 			    GossipChatWindow *window)
 {
-	if (!chat_window_has_toplevel_focus (window)) {
+	if (!gossip_chat_window_has_focus (window)) {
 		window->priv->new_msg = TRUE;
 		chat_window_update_title (window);
 	}
@@ -873,6 +1025,9 @@ chat_window_disconnected_cb (GossipApp        *app,
 
 	priv = window->priv;
 
+	/* FIXME: this works for now, but should operate on a PER
+	   protocol basis not ALL connections since some tabs will
+	   belong to contacts on accounts which might still be online */ 
 	for (l = priv->chats; l != NULL; l = g_list_next (l)) {
 		GossipChat *chat = GOSSIP_CHAT (l->data);
 		chat_window_update_status (window, chat);
@@ -926,10 +1081,12 @@ chat_window_tab_added_cb (GossipNotebook   *notebook,
 	label = chat_window_create_label (window, chat);
 	gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook), child, label);
 
-	g_signal_connect (gossip_app_get_roster (),
-			  "contact_presence_updated",
-			  G_CALLBACK (chat_window_presence_updated_cb),
-			  chat);
+	gtk_notebook_set_tab_label_packing (GTK_NOTEBOOK (notebook), child,
+					    TRUE, TRUE, GTK_PACK_START);
+
+	g_signal_connect (chat, "status_changed",
+			  G_CALLBACK (chat_window_status_changed_cb),
+			  window);
 	g_signal_connect (chat, "name_changed",
 			  G_CALLBACK (chat_window_name_changed_cb),
 			  window);
@@ -958,9 +1115,9 @@ chat_window_tab_removed_cb (GossipNotebook   *notebook,
 	
 	gossip_chat_set_window (chat, NULL);
 
-	g_signal_handlers_disconnect_by_func (gossip_app_get_roster (),
-					      G_CALLBACK (chat_window_presence_updated_cb),
-					      chat);
+	g_signal_handlers_disconnect_by_func (chat,
+					      G_CALLBACK (chat_window_status_changed_cb),
+					      window);
 	g_signal_handlers_disconnect_by_func (chat,
 					      G_CALLBACK (chat_window_name_changed_cb),
 					      window);
@@ -1011,16 +1168,6 @@ chat_window_tabs_reordered_cb (GossipNotebook   *notebook,
 	chat_window_update_menu (window);
 }
 
-static void
-chat_window_close_clicked_cb (GtkWidget  *button,
-			      GossipChat *chat)
-{
-	GossipChatWindow *window;
-
-	window = gossip_chat_get_window (chat);
-	gossip_chat_window_remove_chat (window, chat);
-}
-
 static gboolean
 chat_window_focus_in_event_cb (GtkWidget        *widget,
 			       GdkEvent         *event,
@@ -1032,19 +1179,6 @@ chat_window_focus_in_event_cb (GtkWidget        *widget,
 	return FALSE;
 }
 
-static const gchar *
-chat_window_get_name (GossipChatWindow *window, GossipChat *chat)
-{
-	GossipChatWindowPriv *priv;
-	GossipContact        *contact;
-
-	priv = window->priv;
-
-	contact = gossip_chat_get_contact (chat);
-
-	return gossip_contact_get_name (contact);
-}
-
 static void
 chat_window_drag_data_received (GtkWidget        *widget,
 				GdkDragContext   *context,
@@ -1055,29 +1189,28 @@ chat_window_drag_data_received (GtkWidget        *widget,
 				guint             time, 
 				GossipChatWindow *window)
 {
-	GossipJID        *jid;
-	GossipRosterItem *item;
 	GossipContact    *contact;
 	GossipChat       *chat;
 	GossipChatWindow *old_window;
+	const gchar      *id;
+
+	id = (const gchar*) selection->data;
+	g_print ("Received drag & drop contact from roster with id:'%s'\n", id);
+
+	contact = gossip_session_find_contact (gossip_app_get_session (), id);
 	
-	jid = gossip_jid_new (selection->data);
-	item = gossip_roster_get_item (gossip_app_get_roster (), jid);
-	gossip_jid_unref (jid);
-	
-	if (!item) {
+	if (!contact) {
+		g_print ("No contact found associated with drag & drop\n");
 		return;
 	}
 
-	contact = gossip_roster_get_contact_from_item (gossip_app_get_roster (),
-						       item);
-
-	chat = gossip_chat_get_for_contact (contact, TRUE);
+	chat = GOSSIP_CHAT (gossip_chat_manager_get_chat (gossip_app_get_chat_manager (), contact));
 	old_window = gossip_chat_get_window (chat);
 	
 	if (old_window) {
 		if (old_window == window) {
-			goto finished;
+			gtk_drag_finish (context, TRUE, FALSE, GDK_CURRENT_TIME);
+			return;
 		}
 		
 		gossip_chat_window_remove_chat (old_window, chat);
@@ -1085,7 +1218,6 @@ chat_window_drag_data_received (GtkWidget        *widget,
 
 	gossip_chat_window_add_chat (window, chat);
 
-finished:
 	gtk_drag_finish (context, TRUE, FALSE, GDK_CURRENT_TIME);
 }
 
@@ -1095,7 +1227,7 @@ chat_window_create_notebook (gpointer data)
 	GtkWidget *notebook;
 
 	notebook = gossip_notebook_new ();
-	gtk_notebook_set_scrollable (GTK_NOTEBOOK (notebook), TRUE);
+
 	gtk_widget_show (notebook);
 
 	return notebook;
@@ -1123,6 +1255,14 @@ gossip_chat_window_add_chat (GossipChatWindow *window,
 
 	label = chat_window_create_label (window, chat);
 
+	if (g_list_length (window->priv->chats) == 0) {
+		gint width, height;
+
+		/* first chat, resize the window to its preferred size */
+		gossip_chat_get_geometry (chat, &width, &height);
+		gtk_window_resize (GTK_WINDOW (window->priv->dialog), width, height);
+	}
+
 	gossip_notebook_insert_page (GOSSIP_NOTEBOOK (window->priv->notebook),
 	  			     gossip_chat_get_widget (chat),
 				     label,
@@ -1154,3 +1294,19 @@ gossip_chat_window_switch_to_chat (GossipChatWindow *window,
 	gtk_notebook_set_current_page (GTK_NOTEBOOK (window->priv->notebook),
 				       page_num);
 }
+
+gboolean
+gossip_chat_window_has_focus (GossipChatWindow *window)
+{
+	GossipChatWindowPriv *priv;
+	gboolean              has_focus;
+
+	g_return_val_if_fail (GOSSIP_IS_CHAT_WINDOW (window), FALSE);
+	
+	priv = window->priv;
+
+	g_object_get (priv->dialog, "has-toplevel-focus", &has_focus, NULL);
+
+	return has_focus;
+}
+
