@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2002-2003 Imendio HB
- * Copyright (C) 2003-2004 Geert-Jan Van den Bogaerde <gvdbogaerde@pandora.be>
+ * Copyright (C) 2002-2003 Imendio AB
+ * Copyright (C) 2003-2004 Geert-Jan Van den Bogaerde <geertjan@gnome.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,11 +23,9 @@
 #define __GOSSIP_CHAT_H__
 
 #include <glib-object.h>
-#include <loudmouth/loudmouth.h>
-#include "gossip-app.h"
-#include "gossip-group-chat.h"
-#include "gossip-jid.h"
-#include "gossip-roster.h"
+
+#include "gossip-chat-view.h"
+#include <libgossip/gossip-contact.h>
 
 G_BEGIN_DECLS
 
@@ -43,9 +41,17 @@ typedef struct _GossipChatClass GossipChatClass;
 typedef struct _GossipChatPriv GossipChatPriv;
 
 #include "gossip-chat-window.h"
+#include "gossip-spell.h"
 
 struct _GossipChat {
         GObject         parent;
+
+	/* protected */
+	GossipChatView *view;
+	GtkWidget      *input_text_view;
+	gboolean        is_first_char;
+
+	/* private */
         GossipChatPriv *priv;
 };
 
@@ -53,36 +59,63 @@ struct _GossipChatClass {
         GObjectClass parent;
 
         /* Signals */
-	void (*new_message)      (GossipChat  *chat);
-        void (*composing)        (GossipChat  *chat, 
-			          gboolean     composing);
-	void (*name_changed)     (GossipChat  *chat,
-				  const gchar *name);
+
+	void             (*new_message)       (GossipChat  *chat);
+        void             (*composing)         (GossipChat  *chat, 
+			                       gboolean     composing);
+	void             (*name_changed)      (GossipChat  *chat,
+				               const gchar *name);
+	void             (*status_changed)    (GossipChat  *chat);
+
+	/* vtable */
+
+	const gchar *    (*get_name)           (GossipChat *chat);
+	gchar *          (*get_tooltip)        (GossipChat *chat);
+	GdkPixbuf *      (*get_status_pixbuf)  (GossipChat *chat);
+	GossipContact *  (*get_contact)        (GossipChat *chat);
+	void             (*get_geometry)       (GossipChat *chat,
+			                        gint       *width,
+					        gint       *height);
+	GtkWidget *      (*get_widget)         (GossipChat *chat);
+	gboolean         (*get_group_chat)     (GossipChat *chat);
+
+	gboolean         (*get_show_contacts)  (GossipChat *chat);
+	void             (*set_show_contacts)  (GossipChat *chat,
+						gboolean    show);
 };
 
 GType             gossip_chat_get_type           (void);
-GossipChat *      gossip_chat_get_for_contact    (GossipContact    *contact,
-						  gboolean          create);
-GossipChat *      gossip_chat_get_for_group_chat (GossipContact    *contact,
-						  GossipGroupChat  *g_chat);
-void              gossip_chat_append_message     (GossipChat       *chat,
-						  LmMessage        *message);
 void              gossip_chat_present            (GossipChat       *chat);
-LmHandlerResult   gossip_chat_handle_message     (LmMessage        *message);
 GtkWidget *       gossip_chat_get_widget         (GossipChat       *chat);
 GossipContact *   gossip_chat_get_contact        (GossipChat       *chat);
 void              gossip_chat_set_window         (GossipChat       *chat,
 						  GossipChatWindow *window);
 GossipChatWindow *gossip_chat_get_window         (GossipChat       *chat);
 void              gossip_chat_clear              (GossipChat       *chat);
-gchar *           gossip_chat_get_history        (GossipChat       *chat,
-						  gint              lines);
 void              gossip_chat_scroll_down        (GossipChat       *chat);
 void              gossip_chat_cut                (GossipChat       *chat);
 void              gossip_chat_copy               (GossipChat       *chat);
 void              gossip_chat_paste              (GossipChat       *chat);
+const gchar *     gossip_chat_get_name		 (GossipChat       *chat);
+gchar *           gossip_chat_get_tooltip        (GossipChat       *chat);
+GdkPixbuf *       gossip_chat_get_status_pixbuf  (GossipChat       *chat);
+GtkWidget *       gossip_chat_get_widget         (GossipChat       *chat);
+GossipChatView *  gossip_chat_get_view           (GossipChat       *chat);
+void              gossip_chat_get_geometry       (GossipChat       *chat,
+		                                  int              *width,
+						  int              *height);
+gboolean          gossip_chat_get_group_chat     (GossipChat       *chat);
+gboolean          gossip_chat_get_show_contacts  (GossipChat       *chat);
+void              gossip_chat_set_show_contacts  (GossipChat       *chat,
+						  gboolean          show);
+
+/* for spell checker dialog to correct the misspelled word */
+void              gossip_chat_correct_word       (GossipChat       *chat,
+						  GtkTextIter       start,
+						  GtkTextIter       end,
+						  const gchar      *new_word);
+
 
-        
 G_END_DECLS
 
 #endif /* __GOSSIP_CHAT_H__ */
