@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2003 Imendio HB
+ * Copyright (C) 2003 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,21 +21,26 @@
 #include <config.h>
 #include <gconf/gconf-client.h>
 #include <libgnome/gnome-sound.h>
+#include <libgnome/gnome-triggers.h>
+
+#include <libgossip/gossip-session.h>
+
 #include "gossip-preferences.h"
 #include "gossip-app.h"
 #include "gossip-sound.h"
 
+#define d(x)
+
 extern GConfClient *gconf_client;
 
 void
 gossip_sound_play (GossipSound sound)
 {
-	gboolean     enabled, silent_busy, silent_away;
-	GossipShow   show;
-	const gchar *file;
-	gchar       *str;
+	gboolean             enabled, silent_busy, silent_away;
+        GossipPresence      *p;
+        GossipPresenceState  state;
 
-	if (!gossip_app_is_connected ()) {
+	if (!gossip_session_is_connected (gossip_app_get_session ())) {
 		return;
 	}
 	
@@ -52,39 +57,34 @@ gossip_sound_play (GossipSound sound)
 	if (!enabled) {
 		return;
 	}
-	
-	show = gossip_app_get_show ();
-	
-	if (silent_busy && show == GOSSIP_SHOW_BUSY) {
+
+        p = gossip_session_get_presence (gossip_app_get_session ());
+        state = gossip_presence_get_state (p);
+
+        if (silent_busy && state == GOSSIP_PRESENCE_STATE_BUSY) {
 		return;
 	}
 
-	if (silent_away && (show == GOSSIP_SHOW_AWAY || show == GOSSIP_SHOW_EXT_AWAY)) {
+	if (silent_away && (state == GOSSIP_PRESENCE_STATE_AWAY || state == GOSSIP_PRESENCE_STATE_EXT_AWAY)) {
 		return;
 	}
 
 	switch (sound) {
 	case GOSSIP_SOUND_CHAT:
-		file = "chat1.wav";
+		d(g_print ("Sound: Triggering 'Chat' event.\n"));
+		gnome_triggers_do (NULL, NULL, "gossip", "Chat", NULL);
 		break;
 	case GOSSIP_SOUND_ONLINE:
-		/* Disable for now, it's annoying. */
-		return;
-
-		file = "online.wav";
+		d(g_print ("Sound: Triggering 'Online' event.\n"));
+		gnome_triggers_do (NULL, NULL, "gossip", "Online", NULL);
 		break;
 	case GOSSIP_SOUND_OFFLINE:
-		/* Disable for now, it's annoying. */
-		return;
-
-		file = "offline.wav";
+		d(g_print ("Sound: Triggering 'Offline' event.\n"));
+		gnome_triggers_do (NULL, NULL, "gossip", "Offline", NULL);
 		break;
 	default:
+		d(g_print ("Sound: Unknown sound type.\n"));
 		return;
 	}
-
-	str = g_build_filename (DATADIR "/gossip", file, NULL);
-	gnome_sound_play (str);
-	g_free (str);
 }		
 
